"""Integration tests for the MCP server."""

import os
import tempfile
import pytest
import respx
import httpx
from pathlib import Path
from mcp import ClientSession, StdioServerParameters
from mcp.client.stdio import stdio_client


@pytest.fixture
def simple_openapi_spec():
    """Simple OpenAPI spec for testing."""
    return {
        "openapi": "3.0.0",
        "info": {"title": "Test API", "version": "1.0.0"},
        "servers": [{"url": "https://api.example.com/v1"}],
        "paths": {
            "/users": {
                "get": {
                    "summary": "Get users",
                    "operationId": "getUsers",
                    "responses": {"200": {"description": "Success"}},
                },
            },
            "/users/{userId}": {
                "get": {
                    "summary": "Get user by ID",
                    "operationId": "getUserById",
                    "parameters": [
                        {
                            "name": "userId",
                            "in": "path",
                            "required": True,
                            "schema": {"type": "string"},
                        },
                    ],
                    "responses": {"200": {"description": "Success"}},
                },
            },
        },
    }


@pytest.fixture
def petstore_config_path():
    """Path to petstore test config."""
    return Path(__file__).parent / "fixtures" / "configs" / "petstore.yaml"


@pytest.fixture
def server_params_petstore(petstore_config_path):  # noqa: ANN001
    """Server parameters for petstore config."""
    return StdioServerParameters(
        command="python",
        args=["-m", "mcp_this_openapi", "--config-path", str(petstore_config_path)],
    )


@pytest.mark.asyncio
class TestMCPServerIntegration:
    """Integration tests for the MCP server."""

    async def test_server_starts_with_valid_config(self, simple_openapi_spec):  # noqa: ANN001
        """Test that server starts successfully with a valid config."""
        with respx.mock:
            # Mock the OpenAPI spec fetch
            respx.get("https://api.example.com/openapi.json").mock(
                return_value=httpx.Response(200, json=simple_openapi_spec),
            )

            server_params = StdioServerParameters(
                command="python",
                args=[
                    "-m", "mcp_this_openapi",
                    "--config-path", str(Path(__file__).parent / "fixtures" / "configs" / "petstore.yaml"),  # noqa: E501
                ],
            )

            async with stdio_client(server_params) as (read, write), ClientSession(read, write) as session:  # noqa: E501
                await session.initialize()

                # If we get here without exception, the server started successfully
                assert True

    async def test_list_tools_from_openapi_spec(self, simple_openapi_spec):  # noqa: ANN001
        """Test that tools are properly generated from OpenAPI spec."""
        with respx.mock:
            # Mock the OpenAPI spec fetch
            respx.get("https://petstore3.swagger.io/api/v3/openapi.json").mock(
                return_value=httpx.Response(200, json=simple_openapi_spec),
            )

            server_params = StdioServerParameters(
                command="python",
                args=[
                    "-m", "mcp_this_openapi",
                    "--config-path", str(Path(__file__).parent / "fixtures" / "configs" / "petstore.yaml"),  # noqa: E501
                ],
            )

            async with stdio_client(server_params) as (read, write), ClientSession(read, write) as session:  # noqa: E501
                await session.initialize()
                tools = await session.list_tools()

                # Should have tools generated from the OpenAPI spec
                assert tools.tools
                tool_names = [t.name for t in tools.tools]

                # Check for tools based on the actual Petstore API
                assert any("pet" in name.lower() for name in tool_names)
                assert len(tool_names) > 5  # Should have multiple operations

    async def test_call_tool_with_parameters(self):
        """Test calling a tool with parameters."""
        # Use the real Petstore API to test the tool call
        server_params = StdioServerParameters(
            command="python",
            args=[
                "-m", "mcp_this_openapi",
                "--config-path", str(Path(__file__).parent / "fixtures" / "configs" / "petstore.yaml"),  # noqa: E501
            ],
        )

        async with stdio_client(server_params) as (read, write), ClientSession(read, write) as session:  # noqa: E501
            await session.initialize()

            # Call a Petstore tool (get pet by ID)
            # Try pet ID 1, but handle 404 case (pet might not exist)
            result = await session.call_tool("getPetById", {"petId": 1})

            assert result.content
            result_text = result.content[0].text
            # The real Petstore API might return 404 for pet ID 1, or actual pet data
            assert ("doggie" in result_text or "id" in result_text or
                    "404" in result_text or "not found" in result_text.lower())

    async def test_tool_parameters_schema(self, simple_openapi_spec):  # noqa: ANN001
        """Test that tool parameters are correctly defined from OpenAPI spec."""
        with respx.mock:
            # Mock the OpenAPI spec fetch
            respx.get("https://petstore3.swagger.io/api/v3/openapi.json").mock(
                return_value=httpx.Response(200, json=simple_openapi_spec),
            )

            server_params = StdioServerParameters(
                command="python",
                args=[
                    "-m", "mcp_this_openapi",
                    "--config-path", str(Path(__file__).parent / "fixtures" / "configs" / "petstore.yaml"),  # noqa: E501
                ],
            )

            async with stdio_client(server_params) as (read, write), ClientSession(read, write) as session:  # noqa: E501
                await session.initialize()
                tools = await session.list_tools()

                # Find the getPetById tool
                get_pet_tool = next((t for t in tools.tools if t.name == "getPetById"), None)
                assert get_pet_tool is not None

                # Check parameter schema
                assert get_pet_tool.inputSchema is not None
                assert "properties" in get_pet_tool.inputSchema
                assert "petId" in get_pet_tool.inputSchema["properties"]

                # Check that petId is required
                assert "required" in get_pet_tool.inputSchema
                assert "petId" in get_pet_tool.inputSchema["required"]

    async def test_server_with_authentication(self):
        """Test server with authentication configuration."""
        # Use the real Petstore API with authentication config to test that the server
        # starts successfully and has authentication properly configured
        config_content = """
server:
  name: "auth-test"
openapi:
  spec_url: "https://petstore3.swagger.io/api/v3/openapi.json"
authentication:
  type: "bearer"
  token: "test-token-123"
"""

        with tempfile.NamedTemporaryFile(mode='w', suffix='.yaml', delete=False) as f:
            f.write(config_content)
            temp_config_path = f.name

        try:
            server_params = StdioServerParameters(
                command="python",
                args=["-m", "mcp_this_openapi", "--config-path", temp_config_path],
            )

            async with stdio_client(server_params) as (read, write), ClientSession(read, write) as session:  # noqa: E501
                await session.initialize()

                # Verify the server started with authentication configured
                tools = await session.list_tools()
                assert tools.tools

                # Try calling a tool - it should work (even though the token is fake,
                # the Petstore API may not require authentication for some endpoints)
                tool_names = [t.name for t in tools.tools]
                if "getPetById" in tool_names:
                    result = await session.call_tool("getPetById", {"petId": 1})
                    assert result.content
                    # Don't assert specific content since auth may fail

        finally:
            os.unlink(temp_config_path)

    async def test_server_with_path_filtering(self):
        """Test server with include/exclude path patterns."""
        # Test path filtering using the real Petstore API
        # Include only pet-related endpoints, exclude store-related endpoints
        config_content = """
server:
  name: "filter-test"
openapi:
  spec_url: "https://petstore3.swagger.io/api/v3/openapi.json"
authentication:
  type: "none"
include_patterns: ["^/pet"]
exclude_patterns: ["^/store"]
"""

        with tempfile.NamedTemporaryFile(mode='w', suffix='.yaml', delete=False) as f:
            f.write(config_content)
            temp_config_path = f.name

        try:
            server_params = StdioServerParameters(
                command="python",
                args=["-m", "mcp_this_openapi", "--config-path", temp_config_path],
            )

            async with stdio_client(server_params) as (read, write), ClientSession(read, write) as session:  # noqa: E501
                await session.initialize()
                tools = await session.list_tools()

                tool_names = [t.name for t in tools.tools]

                # Should include /pet paths
                assert any("pet" in name.lower() for name in tool_names)

                # Should exclude /store paths (if any exist in the Petstore API)
                # Note: We'll just verify that filtering worked by checking we have fewer tools
                # than the unfiltered case (which we know has more than 5 tools)
                assert len(tool_names) >= 1  # Should have at least some pet tools
                assert len(tool_names) < 20  # Should be filtered down from the full API

        finally:
            os.unlink(temp_config_path)

    async def test_server_error_handling_invalid_config(self):
        """Test server error handling with invalid configuration."""
        # Create an invalid config (missing required fields)
        config_content = """
server:
  name: "invalid-test"
# Missing openapi section
"""

        with tempfile.NamedTemporaryFile(mode='w', suffix='.yaml', delete=False) as f:
            f.write(config_content)
            temp_config_path = f.name

        try:
            server_params = StdioServerParameters(
                command="python",
                args=["-m", "mcp_this_openapi", "--config-path", temp_config_path],
            )

            # This should fail during server initialization
            with pytest.raises(Exception):  # noqa: PT011
                async with stdio_client(server_params) as (read, write), ClientSession(read, write) as session:  # noqa: E501
                    await session.initialize()

        finally:
            os.unlink(temp_config_path)

    async def test_server_error_handling_network_failure(self):
        """Test server error handling when OpenAPI spec fetch fails."""
        # Create a config that points to a non-existent URL
        config_content = """
server:
  name: "network-failure-test"
openapi:
  spec_url: "https://nonexistent.invalid.domain.example/openapi.json"
authentication:
  type: "none"
"""

        with tempfile.NamedTemporaryFile(mode='w', suffix='.yaml', delete=False) as f:
            f.write(config_content)
            temp_config_path = f.name

        try:
            server_params = StdioServerParameters(
                command="python",
                args=["-m", "mcp_this_openapi", "--config-path", temp_config_path],
            )

            # Should fail when trying to fetch the OpenAPI spec
            with pytest.raises(Exception):  # noqa: PT011
                async with stdio_client(server_params) as (read, write), ClientSession(read, write) as session:  # noqa: E501
                    await session.initialize()

        finally:
            os.unlink(temp_config_path)


@pytest.mark.asyncio
class TestMCPServerWithRealAPIs:
    """Tests with real APIs (these may be skipped in CI)."""

    @pytest.mark.skip(reason="Requires real network access")
    async def test_real_petstore_api(self, petstore_config_path):  # noqa: ANN001
        """Test with the real Petstore API."""
        server_params = StdioServerParameters(
            command="python",
            args=["-m", "mcp_this_openapi", "--config-path", str(petstore_config_path)],
        )

        async with stdio_client(server_params) as (read, write), ClientSession(read, write) as session:  # noqa: E501
            await session.initialize()
            tools = await session.list_tools()

            # Should have tools from the real Petstore API
            assert tools.tools
            tool_names = [t.name for t in tools.tools]

            # These are common Petstore operations
            assert any("pet" in name.lower() for name in tool_names)

    @pytest.mark.skip(reason="Requires GITHUB_TOKEN environment variable")
    async def test_github_api_with_auth(self):
        """Test with GitHub API using authentication."""
        if "GITHUB_TOKEN" not in os.environ:
            pytest.skip("GITHUB_TOKEN environment variable not set")

        github_config_path = Path(__file__).parent / "fixtures" / "configs" / "github.yaml"

        server_params = StdioServerParameters(
            command="python",
            args=["-m", "mcp_this_openapi", "--config-path", str(github_config_path)],
        )

        async with stdio_client(server_params) as (read, write), ClientSession(read, write) as session:  # noqa: E501
            await session.initialize()
            tools = await session.list_tools()

            # Should have filtered tools from GitHub API
            assert tools.tools
            tool_names = [t.name for t in tools.tools]

            # Should include repo and user operations based on our filters
            assert any("repo" in name.lower() for name in tool_names)
            assert any("user" in name.lower() for name in tool_names)

            # Should not include admin operations (excluded by pattern)
            assert not any("admin" in name.lower() for name in tool_names)
